(*
 * cursico
 * Copyright (C) 2008 Fabrice Marchant
 *                           .        @      .
 *                                     orange fr
 * GNU GPL
 *)

open WrapCurses
open WinNkey
open Useful

let used_attr = ref GetOpt.parm_attr

let attr_list_array () =
  Array.of_list (sub_lists !used_attr)

let attr_cases () = Array.length (attr_list_array ())

let loc_of_pair_attr l =
  let loc = loc_of_pair l.pair in
    {loc with
      x = loc.x*(attr_cases ())+array_index l.attr (attr_list_array ())}

let palette_width () = (attr_cases ()) * n_colors

let palette_height = n_colors

let palette_size () = {x = palette_width (); y = palette_height}

let palette_win () = boxed_window (palette_size ()) _00

let palette_key_area () =
  {win = palette_win (); size = palette_size ();
   cursor = {x = (palette_size ()).x - 1; y = 0};
   quit = [tab; 13]}

let cursico draw =
  let draw_win_size = matrix_dim draw in
  let draw_element = subscript_matrix draw in
  let draw_win =
    boxed_window
      draw_win_size
      {x = (1+scr_size.x-draw_win_size.x)/2; y = 2+palette_height} in

  let draw_key_area =
    {win = draw_win; size = draw_win_size;
     cursor = _00;
     quit = [tab]} in
     
    let dka = ref draw_key_area in

    let at_draw_cursor () = draw_element !dka.cursor in
    let set_draw_cursor cna = assign_matrix draw !dka.cursor cna in

    let show_draw () =
      for i = 1 to draw_win_size.x do
        for j = 1 to draw_win_size.y do
          let loc = {x = i; y = j} in
          let cna = draw_element (loc -- _11) in
            WrapCurses.color_print draw_win cna.pair cna.attr loc cna.ch
        done;
      done;
      refresh_cursor !dka in

    let pka = ref (palette_key_area ()) in

    let show_palette () =
      let ch = (at_draw_cursor ()).ch in
        sweep_palette
          (fun loc pair ->
             let w_color_print = WrapCurses.color_print !pka.win pair in
             let ox = 1 + (attr_cases ()) * loc.x in
               Array.iteri
                 (fun x l -> w_color_print l {x = ox+x; y = 1+loc.y} ch)
                 (attr_list_array ()));
        refresh_cursor !pka in

    let pair_of_cursor c = pair_of_loc {c with x = c.x / (attr_cases ())} in
    let attr_of_cursor c = (attr_list_array ()).(c.x mod (attr_cases ())) in
  
    let palette_cursor_pair_attr c =
     {ch = c;
      pair = pair_of_cursor !pka.cursor;
      attr = attr_of_cursor !pka.cursor} in

  set_cursor pka (loc_of_pair_attr (at_draw_cursor ()));

  while true do
    key_loop
      pka
      (fun () -> show_draw (); show_palette ())
      (fun () ->
        set_draw_cursor (palette_cursor_pair_attr (at_draw_cursor ()).ch))
      (fun c -> set_draw_cursor {(at_draw_cursor ()) with ch = c});
            
    key_loop
      dka
      (fun () -> show_palette (); show_draw ())
      (fun () ->
        if !lock then
          let cna = palette_cursor_pair_attr (at_draw_cursor ()).ch in
            set_draw_cursor cna;
            WinNkey.color_print dka cna
        else
          set_cursor pka (loc_of_pair_attr (at_draw_cursor ()))
        )
      (fun c ->
        (let cna = palette_cursor_pair_attr c in
           set_draw_cursor cna;
           WinNkey.color_print dka cna;
           if !dka.cursor.x <> !dka.size.x-1 then
             set_cursor dka (!dka.cursor ++ _10)))
  done