(*
 * cursico
 * Copyright (C) 2008 Fabrice Marchant
 *                           .        @      .
 *                                     orange fr
 * GNU GPL
 *)

open Useful
open WinNkey

let init_ch = ' '

let init_pair = WrapCurses.n_colors-1

let init_attr = []

type afbs = {attr:int list; fg:int; bg:int; str:string}

type token = CR | ESC_STR of afbs | EOF

let input_draw = ref []

let main lexer lexbuf =
  let cur_line = ref [] in
  let rec main' lexer lexbuf =
  let acc_line l = input_draw := (List.rev l)::!input_draw in
    match lexer lexbuf with
      CR ->
        acc_line !cur_line;
        cur_line := [];
        main' lexer lexbuf
    | ESC_STR {attr = a; fg = f; bg = b; str = s} ->
        let a = List.map Ansi.curs_of_esc_attr a in
          for i = 0 to String.length s - 1 do
            cur_line :=
              {ch = String.get s i;
              pair = WrapCurses.pair_of_loc {x = f; y = b};
              WinNkey.attr = a}
            ::!cur_line
          done;      
          main' lexer lexbuf
    | EOF ->
        match !cur_line with
          [] -> ()
        | l -> acc_line l in
    main' lexer lexbuf;
    input_draw := List.rev !input_draw

let default_ch p a s =
   p = init_pair
&& a = init_attr
&& (try
     for i = 0 to (String.length s) - 1 do
       if String.get s i <> init_ch then
         raise Exit
     done;
     true
    with Exit -> false)    

let out_file draw name =  

  let out_chan = open_out name in
  let print = Printf.fprintf out_chan "%s" in
  let out_string cna s =
    let fnb = WrapCurses.loc_of_pair cna.pair in
      print
        (Ansi.attr_fore_back_colour_string
          cna.WinNkey.attr
          fnb.x
          fnb.y
          s) in
  let draw_element = subscript_matrix draw in
  let draw_win_size = matrix_dim draw in
    for j = 1 to draw_win_size.y do
      let acc = ref "" in
      let cur = ref (draw_element {x = 0; y = j-1}) in
      for i = 1 to draw_win_size.x do
        let cna = draw_element ({x = i; y = j} -- _11) in
        let chs = string_of_char cna.ch in
          if cna.pair = !cur.pair && cna.WinNkey.attr = !cur.WinNkey.attr then
            acc := !acc ^ chs
          else(
            out_string !cur !acc;
            acc := chs;
            cur := cna)
      done;
      (match !acc with
        "" -> ()
      | s when default_ch !cur.pair !cur.WinNkey.attr s -> ()
      | s -> out_string !cur s);
      if j <> draw_win_size.y then
        print "\n"
    done;
  close_out out_chan
