{
  open Useful
  open EscIO

  let get_attr a =
    let acc = ref [] in
      for i = String.length a /2-1 downto 0 do
        acc := int_of_digit (String.get a (2*i))::!acc
      done;
      !acc

  let file =
    (match !GetOpt.file_spec with
      None -> "default.esc"
    | Some name -> name)
(*
  let cur_attr = ref ([]:int list)

  let cur_fg = ref Curses.Color.white

  let cur_fg = ref Curses.Color.black
*)
}

let esc = '\027'

let digit = ['0'-'9']

let attr = digit ';'

let str = [^'\027']*

let str_ncr = [^'\027' '\n']*

(*
  Used escaped sequences belongs to subset of possible ones. They follow this
  form :
  esc '[' attr* '3' digit ';' '4' digit 'm' str esc "[0m"
*)

rule token = parse
  '\n' { CR }
| esc '['
  (attr* as a)
  '3' (digit as f)
  ';' '4' (digit as b)
  'm'
  (str as s)
  esc "[0m"
    {ESC_STR
      {attr = get_attr a;
      fg = int_of_digit f;
      bg = int_of_digit b;
      str = s}}
| str_ncr as s
    {ESC_STR
      {attr = [];
      fg = Curses.Color.white;
      bg = Curses.Color.black;
      str = s}}
| eof { EOF }
| _   { failwith
        ("File "
       ^ file
       ^ ": Error at line "
       ^ string_of_int (succ (List.length !input_draw))) }

{
(*
  Use
    if specified dim_parms then
      them
    else
      if an existing file_parm then
        file_dim
      else
        default_dim
*)
  let default_dim = {x = 40; y = 10} in

  let file_dim =
    (try
      let chan = open_in file in
        main token (Lexing.from_channel chan);
        close_in chan;
          Some {x = max 1
                      (List.fold_left
                        max
                        0
                        (List.map List.length !EscIO.input_draw));
                y = max 1 (List.length !EscIO.input_draw)}
     with Sys_error _ -> None) in

  let draw_win_size =
    match !GetOpt.dim, file_dim with
      Some (w, h), _ -> {x = w; y = h}
    | None, None -> default_dim
    | None, Some d -> d in

  let draw =
    matrix
      draw_win_size
      {WinNkey.ch = init_ch;
      WinNkey.pair = init_pair;
      WinNkey.attr = init_attr} in

  (match file_dim with
    None -> ()
  | Some _ ->
      let fa = Array.of_list (List.map Array.of_list !input_draw) in
        for y = 0 to (min draw_win_size.y (Array.length fa)) - 1  do
          for x = 0 to (min draw_win_size.x (Array.length fa.(y))) - 1 do
            let cna = fa.(y).(x) in
              assign_matrix draw {x = x; y = y} cna;
              Cursico.used_attr :=
                list_union !Cursico.used_attr cna.WinNkey.attr
          done
        done);

  (try
    Cursico.cursico draw;
  with
    WinNkey.Exit_loop save ->
      match save with
        true -> EscIO.out_file draw file
      | false -> ());
  WrapCurses.exit_program "Bye !"
}