(*
 * cursico
 * Copyright (C) 2008 Fabrice Marchant
 *                           .        @      .
 *                                     orange fr
 * GNU GPL
 *)

let string_of_char = String.make 1

let int_of_digit d = (int_of_char d) - (int_of_char '0')

let is_char i = 0 < i && i < 256

let maybe_char k =
  if is_char k then
    Some (char_of_int k)
  else
    None

let cr = int_of_char '\n'

let tab = int_of_char '\t'

let beep () = Printf.printf "%c" '\007'

(*****************************************************************************)

let ( <<- ) f g x = f (g x)

let toggle r = r := not !r

let rec sub_lists =
  function
      [] -> [[]]
    | h::t ->
  let s = sub_lists t in
    List.fold_left (fun a l -> (h::l)::a) s s

let list_union l =
    List.rev
    <<- (List.fold_left (fun a e -> if List.mem e a then a else e::a)
         <<- List.rev) l

exception Exit_int of int

let array_index e a =
  try
    for i = 0 to Array.length a - 1 do
      if a.(i) = e then
        raise (Exit_int i)
    done;
    raise Not_found
  with (Exit_int v) -> v

(*****************************************************************************)
let z m x =
  Printf.printf "<%s=%i>" m x;
  flush stdout

type int_vec2 = {x:int; y:int}

let matrix dim = Array.make_matrix dim.x dim.y

let subscript_matrix m (s:int_vec2) = m.(s.x).(s.y)

let assign_matrix m (s:int_vec2) v = m.(s.x).(s.y) <- v

let matrix_dim m = {x = Array.length m; y = Array.length m.(0)}

let ( -- ) v v' = {x = v.x-v'.x; y = v.y-v'.y}

let neg v = {x = -v.x; y = -v.y}

let ( ++ ) v v' = {x = v.x+v'.x; y = v.y+v'.y}

let _00 = {x = 0; y = 0}
let _11 = {x = 1; y = 1}
let _10 = {x=1; y=0}
and _01 = {x=0; y=1}

let iv2_print v =
  Printf.printf "(x=%i, y=%i)" v.x v.y(*;
  flush stdout*)