(*
 * cursico
 * Copyright (C) 2008 Fabrice Marchant
 *                           .        @      .
 *                                     orange fr
 * GNU GPL
 *)

open WrapCurses
open Useful

type t =
  {win:Curses.window; size:int_vec2; cursor:int_vec2; quit:int list}

let curs_loc ka = _11 ++ ka.cursor

let refresh_cursor ka =
  wmove ka.win (curs_loc ka);
  wrefresh ka.win

let set_cursor kar loc =
  kar := {!kar with cursor = loc}

type char_and_attr = {ch:char; pair:int; attr:int list}

let color_print kar cna =
  color_print !kar.win cna.pair cna.attr (curs_loc !kar) cna.ch

let mov_curs kar rfrsh new_loc =
  wmove !kar.win new_loc;
  kar := {!kar with cursor = new_loc -- _11 };
  rfrsh ()

(*
let mov_curs_delta2 kar rfrsh delta_curs =
  mov_curs kar rfrsh ((curs_loc !kar) ++ delta_curs)
*)

let mov_curs_delta kar rfrsh delta_curs =
    kar := {!kar with cursor = !kar.cursor ++ delta_curs};
    wmove !kar.win (curs_loc !kar);
    rfrsh ()

let lock = ref false

let state_text = [|
  "Arrows/Home/End: move";
  "Tab:             focus";
  "F1:              lock";
  "F9:              quit";
  "F10:             quit & save"|]

let state_win_size =
  {x = Array.fold_left (fun a e -> max a (String.length e)) 0 state_text;
   y = Array.length state_text}

let state_win =
  boxed_window
    state_win_size
    {x = scr_size.x-(1+state_win_size.x+1); y = 0}

let state_win_print =
  WrapCurses.color_print_string
    state_win
    (n_colors-1)
    []

let _ =
  Array.iteri
    (fun i -> state_win_print {x = 1; y = succ i})
    state_text;
  wrefresh state_win

exception Exit_loop of bool

let key_loop kar rfrsh mov_update' handle_char' =

    let mov_update () = mov_update' (); rfrsh () in
    let handle_char c = handle_char' c; rfrsh () in

    mov_update ();
    try
      while true do
        let cursor = !kar.cursor in
        let max = !kar.size -- _11 in
          (match getchar () with
             k when List.mem k !kar.quit -> raise Exit
           | k when k = Curses.Key.f 10 -> raise (Exit_loop true)
           | k when k = Curses.Key.f 9  -> raise (Exit_loop false)
           | k when k = Curses.Key.f 1 ->
               toggle lock;
               state_win_print
                 {x = 5; y = 3}
                 (if !lock then
                   "*"
                  else
                   " ");
                wrefresh state_win;
                rfrsh ()
           | k when k = cr -> ()
           | k when (match maybe_char k with
                       None -> false
                     | Some c -> handle_char c; true) -> ()           
           | k when k = Curses.Key.up && cursor.y > 0 ->
               mov_curs_delta kar mov_update (neg _01)
           | k when k = Curses.Key.down && cursor.y < max.y ->
               mov_curs_delta kar mov_update _01
           | k when k = Curses.Key.left && cursor.x > 0 ->
               mov_curs_delta kar mov_update (neg _10)
           | k when k = Curses.Key.right && cursor.x < max.x ->
               mov_curs_delta kar mov_update _10
           | k when k = Curses.Key.home ->
               mov_curs kar mov_update (_11 ++ {x = 0; y = cursor.y})
           | k when k = Curses.Key.end_ ->
               mov_curs
                 kar
                 mov_update
                 (_11 ++ {x = max.x; y = cursor.y})
           | k -> ())
      done
    with Exit -> ()