(*
 * cursico
 * Copyright (C) 2008 Fabrice Marchant
 *                           .        @      .
 *                                     orange fr
 * GNU GPL
 *)

open Curses
open Useful

let no_char = -1

let getchar () =
  try
    while true do
      let i = getch () in
        if i <> no_char then
          raise (Exit_int i)
    done;
    raise Exit (* Not reached *)
  with (Exit_int v) -> v

let exit_program message =
  endwin ();
  Printf.printf "%s\n" message;
  exit 0

let boxed_window inside_size loc =
  let outside_size = _11 ++ inside_size ++ _11 in
  let win = newwin outside_size.y outside_size.x loc.y loc.x in
    box win 0 0;
    win

let scr_size,
  n_colors =
  let std_scr = initscr () in
    assert ( start_color () );
    ignore ( curs_set 1 ); (* Visible cursor *)
    assert ( keypad std_scr true );
    ignore ( nodelay std_scr true );
    nonl ();
    assert( cbreak () );
    assert( noecho () );

    ignore ( getch () ); (* Need this to be ready to print !? *)
    ignore ( getch () );
    ignore ( getch () );

    let w, h = getmaxyx std_scr in
      {x = w; y = h},
  colors ()

let pair_of_loc loc = loc.x + loc.y * n_colors

let loc_of_pair pair = {x = pair mod n_colors; y = pair / n_colors}

let sweep_palette f =
  let max_colors = n_colors-1 in
    for fg = 0 to max_colors do
      for bg = 0 to max_colors do
        let loc = {x = fg; y = bg} in
          f loc (pair_of_loc loc)
      done;
    done

let _ =
  sweep_palette
    (fun loc pair ->
       if pair <> 0 then
         assert ( init_pair pair loc.x loc.y ))

let wmove win loc = assert (wmove win loc.y loc.x)

let color_print win pair attr_list loc ch =
  let attr_on w p = wattron win (A.color_pair pair) in
    attr_on win pair;
    assert
     (mvwaddch
        win
        loc.y
        loc.x
        (List.fold_left (lor) (int_of_char ch) attr_list))

let color_print_string win pair attr_list loc s =
  for i = 0 to String.length s -1 do
    color_print
      win
      pair
      attr_list
      {loc with x = loc.x + i}
      (String.get s i)
  done

let wrefresh win = assert (wrefresh win)